<?php

namespace Drupal\sso_connector\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\sso_connector\Services\JwtTokenValidator;
use Drupal\sso_connector\Services\UserSyncServiceInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for handling the return path from the Identity Provider.
 */
class SsoController extends ControllerBase {

  /**
   * The logger channel factory.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The JWT token validator service.
   *
   * @var \Drupal\sso_connector\Services\JwtTokenValidator
   */
  protected $jwtTokenValidator;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;


  /**
   * The configuration object for sso_connector settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The user synchronization service.
   *
   * @var \Drupal\sso_connector\Services\UserSyncServiceInterface
   */
  protected $userSync;

  /**
   * Constructs a new SsoController.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory.
   * @param \Drupal\sso_connector\Services\JwtTokenValidator $jwt_token_validator
   *   The JWT token validator service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\sso_connector\Services\UserSyncServiceInterface $user_sync
   *   The user synchronization service.
   */
  public function __construct(
    LoggerChannelFactoryInterface $loggerFactory,
    JwtTokenValidator $jwt_token_validator,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    UserSyncServiceInterface $user_sync,
  ) {
    $this->logger = $loggerFactory->get('sso_connector');
    $this->jwtTokenValidator = $jwt_token_validator;
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory->get('sso_connector.settings');
    $this->userSync = $user_sync;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory'),
      $container->get('sso_connector.jwt_token_validator'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('sso_connector.user_sync')
    );
  }

  /**
   * Handles the return from the Identity Provider.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response to send back to the browser.
   */
  public function fromIdp(Request $request) {
    $idpUrl = $this->config->get('idp_url');
    $token = $request->query->get('token');

    try {
      if (!$token) {
        $this->logger->warning('SSO attempt without a token from IP {ip}.', ['ip' => $request->getClientIp()]);
        return new TrustedRedirectResponse($idpUrl . '/user/login');
      }

      $validationResult = $this->jwtTokenValidator->validate($token);
      if (!$validationResult['valid']) {
        $this->logger->notice('Invalid token received from IP {ip}. Token: {token}', [
          'ip' => $request->getClientIp(),
          'token' => $token,
        ]);
        return new TrustedRedirectResponse($idpUrl . '/user/login?error=invalid_token');
      }

      $userMail = $validationResult['data']['mail'];
      $response = $this->loginUserByEmail($userMail, $validationResult);
      if ($response) {
        return $response;
      }

      // If user sync and login attempts failed, handle as an error.
      $this->logger->critical('Failed to synchronize or log in the user with email {email}, redirecting to error.', ['email' => $userMail]);
      return $this->handleLoginFailure($idpUrl);
    }
    catch (\Exception $e) {
      $this->logger->error('Unexpected error during SSO process: {message}', ['message' => $e->getMessage()]);
      return new TrustedRedirectResponse($idpUrl . '/user/login?error=unexpected_error');
    }
  }

  /**
   * Empty returning path.
   */
  public function returnPath(Request $request) {
    return [];
  }

  /**
   * Attempts to load a user by email and finalize the login.
   *
   * This method loads a user based on the provided email, updates the user's
   * data if necessary, and finalizes the user login. If no user is found or
   * if the login process cannot be finalized, the method returns NULL.
   *
   * @param string $userMail
   *   The user's email address.
   * @param array $validationResult
   *   Validation data including user details.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse|null
   *   Returns a redirect response to the front page upon successful login,
   *   or NULL if the user cannot be loaded or logged in.
   */
  protected function loginUserByEmail($userMail, $validationResult) {
    $users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['mail' => $userMail]);
    $user = reset($users);
    $userSync = $this->config->get('user_sync');
    if ($user == NULL && $userSync) {
      $this->userSync->synchronizeUser($validationResult['data']);
      $users = $this->entityTypeManager->getStorage('user')
        ->loadByProperties(['mail' => $userMail]);
      $user = reset($users);
    }
    if ($user instanceof UserInterface) {
      if ($userSync) {
        $this->userSync->updateUserData($user, $validationResult['data']);
        $this->logger->info('User data updated for user {user_id} during SSO.', ['user_id' => $user->id()]);
      }
      user_login_finalize($user);
      return $this->redirect('<front>');
    }
    else {
      $this->logger->error('User not found or failed to load during SSO with email {email}.', ['email' => $userMail]);
    }

    return NULL;
  }

  /**
   * Handles unsuccessful login attempts and redirects.
   *
   * This method is called when a user login attempt fails, either because
   * the user could not be loaded or synchronized properly.
   *
   * @param string $idpUrl
   *   The URL to redirect to on error.
   *
   * @return \Drupal\Core\Routing\TrustedRedirectResponse
   *   Returns a response that redirects the user to the identity prsovider's
   *   login or error page, indicating a problem with the login process.
   */
  protected function handleLoginFailure($idpUrl) {
    return new TrustedRedirectResponse($idpUrl . '/?error=user_sync_error');
  }

}
